/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */

#CalendarButton {
     background-color: transparent;
}
#CalendarButton:hover {
     border-width: 1px;
     border-color: rgba(255,255,255,0.1); 
     background: rgba(255,255,255,0.2);
}
#CalendarButtonClicked {
     border-width: 1px;
     border-color: rgba(255,255,255,0.2); 
     background-color: rgba(0,0,0,0.1);
}
#CalendarButtonClicked:hover {
     background: rgba(255,255,255,0.5);
}
#CalendarButtonRelease {
     background-color: transparent;
}
#CalendarButtonRelease:hover {
     border-width: 1px;
     border-color: rgba(255,255,255,0.1); 
     background: rgba(255,255,255,0.2);
}
